﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Unity.Collections;
using UnityEngine;

public class TerrainMaster : MonoBehaviour
{
    [Header("User settings")]
    public List<Octave> octaves;

    public enum GenerationMode
    {
        Threads, Jobs
    }

    public GenerationMode mode;

    [Min(0)]
    [Tooltip("Minimum number of neighboring terrains.")]
    public int neighboringCount = 1;

    public Vector2 initialCrawlPosition;

    [Tooltip("How tall the terrain is. May be changed to allow for higher terrains.")]
    public float terrainHeight = 600;
    
    [SerializeField]
    private bool generateNewCrawlPosition;

    [SerializeField]
    private bool refresh;

    [Header("Autogenerated values")]
    public Transform player;

    public Dictionary<Vector2Int, TerrainChunker> chunkers = new();

    public Vector2Int currentGridPosition;

    private void Start()
    {
        GameObject po = GameObject.FindGameObjectWithTag("Player");
        player = po.transform;

        int n = neighboringCount;

        for (int x = -n; x <= n; x++)
        {
            for (int y = -n; y <= n; y++)
            {
                Vector2Int gridPos = new(x, y);
                GameObject chunkGO = new();
                TerrainChunker chunker;

                switch (mode)
                {
                    case GenerationMode.Threads:
                        chunker = chunkGO.AddComponent<TerrainChunkerThreads>();
                        break;
                    case GenerationMode.Jobs:
                    default:
                        chunker = chunkGO.AddComponent<TerrainChunkerJobs>();
                        break;
                }

                chunker.transform.parent = transform;
                chunker.Initialize(this, gridPos, initialCrawlPosition);
                chunkers[gridPos] = chunker;
            }
        }
    }

    public void UpdateGridPosition()
    {
        // Check player position.
        float gridSize = chunkers.Values.First().GetTerrainSize();
        Vector3 pPos = player.position - (Vector3.one * gridSize / 2);
        Vector2Int pGridPos = new(
            Mathf.RoundToInt(pPos.x / gridSize),
            Mathf.RoundToInt(pPos.z / gridSize)
        );
        Vector2Int cGridPos = currentGridPosition;

        if (pGridPos != cGridPos)
        {
            print($"Player grid pos: {pGridPos}, player actual: {pPos}");
            Vector2Int gridPosDiff = pGridPos - currentGridPosition;
            currentGridPosition = pGridPos;

            int n = neighboringCount;

            HashSet<Vector2Int> toUpdate = new();

            if (gridPosDiff.x != 0)
            {
                int xSign = (int)Mathf.Sign(gridPosDiff.x);
                // Add top/bottom rows
                for (int y = -n; y <= n; y++)
                    toUpdate.Add(new(n * xSign, y));
            }
            if (gridPosDiff.y != 0)
            {
                int ySign = (int)Mathf.Sign(gridPosDiff.y);
                // Add top/bottom rows
                for (int x = -n; x <= n; x++)
                    toUpdate.Add(new(x, n * ySign));
            }

            foreach (var localGridPos in toUpdate)
            {
                Vector2Int globalGridPos = localGridPos + currentGridPosition;
                Vector2Int keyPos = globalGridPos.Bound(-n, n);

                chunkers[keyPos].UpdateChunk(globalGridPos);
            }

            //for (int x = -n; x <= n; x++)
            //{
            //    for (int y = -n; y <= n; y++)
            //    {
            //        Vector2Int keyPos = new(x, y);
            //        Vector2Int gridPos = keyPos + currentGridPosition;

            //        chunkers[keyPos].UpdateChunk(gridPos);
            //    }
            //}
        }
    }
    
    private void FixedUpdate()
    {
        UpdateGridPosition();
    }
    
    private void OnValidate()
    {
        if (generateNewCrawlPosition)
        {
            generateNewCrawlPosition = false;
            initialCrawlPosition = RNG.GetRandomVector2(-2500, 2500);
        }

        if (refresh && Application.isPlaying)
        {
            refresh = false;

            foreach (var chunker in chunkers.Values)
            {
                chunker.UpdateChunk(chunker.gridPosition, true);
            }
        }
    }
}